@@ -8,7 +8,6 @@
 //  Modified by Hua Zhong on 6/28/2015. Increased float comparison precision
 
 #include "ExactFunctionalTest.h"
-
 // #include <Rcpp.h>//for deleted
 // using namespace Rcpp;//for deleted
 
@@ -39,39 +38,7 @@ bool ll (const mydouble & a, const mydouble & b){ // a<b?
   return false;
 }
 ////
-  
-mydouble funchisq(const vector<vector<int> > & O, const vector<int> & rowsums,
-                const vector<int> & colsums, int n)
-{
-    mydouble fc = 0.0;
-    
-    if (n == 0 || O.size() == 0) {
-        return fc;
-    } else if(O[0].size() == 0) {
-        return fc;
-    }
-    
-    size_t nrows = O.size();  // number of rows
-    size_t ncols = O[0].size();  // number of columns
-    
-    mydouble ej = n / (mydouble) ncols;
-    if(ej>0){
-        for (size_t j=0; j<ncols; ++j) {
-            fc -= (colsums[j] - ej) * (colsums[j] - ej) / ej;
-        }
-    }
-    
-    for (size_t i=0; i<nrows; ++i) {
-        // Expected cound for cell (i,j):
-        mydouble eij = rowsums[i] / (mydouble) ncols;
-        if (eij > 0) {
-            for (size_t j=0; j<ncols; ++j) {
-                fc += (O[i][j] - eij) * (O[i][j] - eij) / eij;
-            }
-        }
-    }
-    return fc;
-}
+
 
 mydouble upper_bound(const vector<vector<int> > &A, size_t i,
                    const mydouble A_running_stat,
@@ -83,9 +50,9 @@ mydouble upper_bound(const vector<vector<int> > &A, size_t i,
 {
     // return 10e10;
     mydouble upper_bound = A_running_stat;
-    
+
     vector<int> U(O_colsums);
-    
+
     size_t ncols = A[0].size(); // Number of colmns
     size_t nrows = A.size(); // Number of rows
     if (i > 0) {
@@ -93,16 +60,16 @@ mydouble upper_bound(const vector<vector<int> > &A, size_t i,
             U[q] = O_colsums[q] - A_running_colsums[i-1][q];
         }
     }
-    
+
     vector<size_t> order( ncols );
     for (size_t q=0; q<ncols; ++q) {
         order[q] = q;
     }
-    
+
     // sort U in decreasing order
     sort(order.begin(), order.end(),
          [&U](size_t i1, size_t i2) {return U[i1] > U[i2];});
-    
+
     for (size_t l=i; l<nrows; ++l) {
         // find lower bound for row l
         int runsum = 0;
@@ -127,9 +94,9 @@ mydouble upper_bound(const vector<vector<int> > &A, size_t i,
             }
         }
     }
-    
+
     return upper_bound;
-    
+
 }
 
 mydouble upper_bound(const vector<vector<int> > &A, size_t i, size_t j,
@@ -142,9 +109,9 @@ mydouble upper_bound(const vector<vector<int> > &A, size_t i, size_t j,
 {
     // return 10e10;
     mydouble upper_bound = A_running_stat;
-    
+
     vector<int> U(O_colsums);
-    
+
     size_t ncols = A[0].size(); // Number of colmns
     size_t nrows = A.size(); // Number of rows
     if (i > 0) {
@@ -155,36 +122,36 @@ mydouble upper_bound(const vector<vector<int> > &A, size_t i, size_t j,
                 U[q] = O_colsums[q] - A_running_colsums[i-1][q];
             }
         }
-        
+
         /*
          for (size_t q=0; q<ncols; ++q) {
          U[q] = O_colsums[q] - A_running_colsums[i-1][q];
          } */
     }
-    
+
     vector<size_t> order( ncols );
     for (size_t q=0; q<ncols; ++q) {
         order[q] = q;
     }
-    
+
     // sort U in decreasing order
     sort(order.begin(), order.end(),
          [&U](size_t i1, size_t i2) {return U[i1] > U[i2];});
-    
+
     for (size_t l=i; l<nrows; ++l) {
         // find lower bound for row l
-        
+
         if (l == i && j > 0) {
-            
+
             vector<size_t> ord( ncols );
             for (size_t q=0; q<ncols; ++q) {
                 ord[q] = q;
             }
-            
+
             // sort U in decreasing order
             sort(ord.begin()+j, ord.end(),
                  [&U](size_t i1, size_t i2) {return U[i1] > U[i2];});
-            
+
             // find upper bound for row l
             int runsum = A_running_rowsums[i][j-1];
             mydouble el = O_rowsums[l] / (mydouble) ncols;
@@ -209,7 +176,7 @@ mydouble upper_bound(const vector<vector<int> > &A, size_t i, size_t j,
             }
             continue;
         }
-        
+
         int runsum = 0;
         mydouble el = O_rowsums[l] / (mydouble) ncols;
         for (size_t k=0; k<ncols; ++k) {
@@ -232,9 +199,9 @@ mydouble upper_bound(const vector<vector<int> > &A, size_t i, size_t j,
             }
         }
     }
-    
+
     return upper_bound;
-    
+
 }
 
 mydouble lower_bound(const vector<vector<int> > &A, size_t i, size_t j,
@@ -246,12 +213,12 @@ mydouble lower_bound(const vector<vector<int> > &A, size_t i, size_t j,
                    mydouble O_stat)
 {
     mydouble lower_bound = A_running_stat;
-    
+
     vector<int> U(O_colsums);
-    
+
     size_t nrows = A.size();
     size_t ncols = A[0].size();
-    
+
     if (i > 0) {
         for (size_t q=0; q<ncols; ++q) {
             if (q < j) {
@@ -261,16 +228,16 @@ mydouble lower_bound(const vector<vector<int> > &A, size_t i, size_t j,
             }
         }
     }
-    
+
     vector<size_t> order( ncols );
     for (size_t q=0; q<ncols; ++q) {
         order[q] = q;
     }
-    
+
     // sort U in increasing order
     sort(order.begin(), order.end(),
          [&U](size_t i1, size_t i2) {return U[i1] < U[i2];});
-    
+
     for (size_t l=i; l<nrows; ++l) {
         // find lower bound for row l
         int runsum = 0;
@@ -290,7 +257,7 @@ mydouble lower_bound(const vector<vector<int> > &A, size_t i, size_t j,
             }
         }
     }
-    
+
     return lower_bound;
 }
 
@@ -303,22 +270,22 @@ mydouble prob_entire_branch(vector<vector<int> > &A,
                                const vector<int> & O_colsums)
 {
     mydouble prob = A_running_prob;
-    
+
     if(j != 0) throw "ERROR: can only compute whole rows";
-    
+
     if (i == 0) {
         prob = 1.0;
     } else {
         int remaing_rowsum=0;
         size_t nrows = A.size();
         size_t ncols = A[0].size();
-        
+
         for (size_t l=i; l<nrows; ++l) {
             prob /= factorial<mydouble>(O_rowsums[l]);
             remaing_rowsum += O_rowsums[l];
         }
         prob *= factorial<mydouble>(remaing_rowsum);
-        
+
         for(size_t q=0; q<ncols; q++) {
             prob /= factorial<mydouble>(O_colsums[q] - A_running_colsums[i-1][q]);
         }
@@ -352,85 +319,85 @@ mydouble enumerate_next (vector<vector<int> > &A,
                             )
 {
     mydouble prob=0.0;
-    
+
     size_t nrows = A.size();
     size_t ncols = A[0].size();
-    
+
     size_t j_next = j+1;
     size_t i_next = i;
     if(j_next == ncols) {
         j_next = 0;
         i_next += 1;
     }
-    
+
     int Lij;
     int Uij;
-    
+
     if (i == nrows - 1) { // last row
-        
+
         Lij = O_colsums[j] - A_running_colsums[i-1][j];
-        
+
     } else if (j == ncols - 1) { // last column
-        
+
         Lij = O_rowsums[i] - A_running_rowsums[i][j-1];
-        
+
     } else {
-        
+
         Lij = 0;
-        
+
     }
-    
+
     Uij = min(O_rowsums[i] - (j > 0 ? A_running_rowsums[i][j-1]:0),
               O_colsums[j] - (i > 0 ? A_running_colsums[i-1][j]:0));
-    
-    
+
+
     mydouble eij = O_rowsums[i] / (mydouble) ncols;
-    
+
     // mydouble A_running_stat_before = A_running_stat;
-    
+
     for(int x=Lij; x <= Uij; x++) {
-        
+
         A[i][j] = x;
-        
+
         if (A[i][j] == Lij) {
             A_running_prob /= factorial<mydouble>( Lij );
         } else {
             A_running_prob /= A[i][j];
         }
-        
+
         // update running statistics
         A_running_rowsums[i][j] = (j > 0 ? A_running_rowsums[i][j-1] : 0) + A[i][j];
         A_running_colsums[i][j] = (i > 0 ? A_running_colsums[i-1][j] : 0) + A[i][j];
-        
+
         mydouble d = A[i][j] - eij;
         mydouble stat_ij = 0;
         if(eij>0) stat_ij = d * d / eij;
-        
+
         // A_running_stat += stat_ij;
-        
+
         prob += (*traverse)
         (A, i_next, j_next, A_running_stat + stat_ij, A_running_prob,
          A_running_rowsums, A_running_colsums,
          O_rowsums, O_colsums, O_stat, lb_method, ub_method);
-        
+
         // BEGIN ***
         // NUMERICALLY VERY CRITICAL. Must use option 1 instead of 2
         // Option 1:
         //   A_running_stat = A_running_stat_before;
         // Option 2: A_running_stat -= stat_ij;
         // Option 3: No need to change A_running_stat
-        
+
         // END ***
-        
+
         // A_running_prob *= fac_ij;
-        
+
         // restore running statistics
         A_running_rowsums[i][j] = 0; // -= A[i][j];
         A_running_colsums[i][j] = 0; // -= A[i][j];
         A[i][j] = 0;
-        
+
     }
-    
+
     return prob;
 }
 
@@ -449,9 +416,9 @@ mydouble traverse_ge_observed_stat
  )
 {
     mydouble prob=0.0;
-    
+
     size_t nrows = A.size();
-    
+
     if(i >= nrows) { // calculate probability of A
       prob = ge(A_running_stat, O_stat) ? A_running_prob : 0.0;
         //prob = (A_running_stat >= O_stat || is_close(A_running_stat, O_stat)) ? A_running_prob : 0.0;
@@ -462,7 +429,7 @@ mydouble traverse_ge_observed_stat
          O_rowsums, O_colsums, O_stat)
          < O_stat)
          { // check the upper bound on stat
-         
+
          // cout << "Skip entire branch" << endl;
          //cout << "0";
          prob = 0.0;
@@ -473,18 +440,18 @@ mydouble traverse_ge_observed_stat
                               O_rowsums, O_colsums, O_stat)
                , O_stat))
     { // check the upper bound on stat
-        
+
         // cout << "Skip entire branch" << endl;
         //cout << "0";
         prob = 0.0;
-        
-        
+
+
     } else if (lb_method == LBON && j == 0 && ge(A_running_stat, O_stat)) {
-        
+
         // cout << "Keep entire branch" << endl;
         prob = prob_entire_branch(A, i, j, A_running_prob, A_running_rowsums,
                                   A_running_colsums, O_rowsums, O_colsums);
-        
+
         //cout << "e";
     } else if (lb_method == LBON && j == 0 &&
                ge(lower_bound(A, i, j, A_running_stat,
@@ -492,15 +459,15 @@ mydouble traverse_ge_observed_stat
                            O_rowsums, O_colsums, O_stat)
                , O_stat))
     { // check lower bound on stat
-        
+
         // cout << "Keep entire branch" << endl;
         prob = prob_entire_branch(A, i, j, A_running_prob, A_running_rowsums,
                                   A_running_colsums, O_rowsums, O_colsums);
-        
+
         //cout << "+";
-        
+
     } else {
-        
+
         prob = enumerate_next(A, i, j, A_running_stat, A_running_prob,
                               A_running_rowsums, A_running_colsums,
                               O_rowsums, O_colsums, O_stat,
@@ -525,19 +492,19 @@ mydouble traverse_lt_observed_stat
  )
 {
     mydouble prob=0.0;
-    
+
     size_t nrows = A.size();
-    
+
     if(i >= nrows) { // calculate probability of A
-        
+
         prob = ll(A_running_stat, O_stat) ? A_running_prob : 0.0;
-        
+
         /*    } else if (ub_method == UB_BY_ELE // && j == 0
          && upper_bound(A, i, j, A_running_stat,
          A_running_rowsums, A_running_colsums,
          O_rowsums, O_colsums, O_stat)
          < O_stat) { // check the upper bound on stat
-         
+
          // cout << "Keep entire branch" << endl;
          prob = prob_entire_branch(A, i, j, A_running_prob, A_running_rowsums,
          A_running_colsums, O_rowsums, O_colsums);
@@ -547,36 +514,36 @@ mydouble traverse_lt_observed_stat
                               A_running_rowsums, A_running_colsums,
                               O_rowsums, O_colsums, O_stat)
                , O_stat)) { // check the upper bound on stat
-        
+
         // cout << "Keep entire branch" << endl;
         // prob = 0.0;
         prob = prob_entire_branch(A, i, j, A_running_prob, A_running_rowsums,
                                   A_running_colsums, O_rowsums, O_colsums);
-        
+
     } else if (lb_method == LBON && j == 0 && ge(A_running_stat, O_stat)) {
-        
+
         // cout << "Skip entire branch" << endl;
         prob = 0.0;
-        
+
         //cout << "e";
     } else if (lb_method == LBON && j == 0 &&
                ge(lower_bound(A, i, j, A_running_stat,
                            A_running_rowsums, A_running_colsums,
                            O_rowsums, O_colsums, O_stat)
                , O_stat)) { // check lower bound on stat
-        
+
         // cout << "Skip entire branch" << endl;
         prob = 0.0;
-        
+
     } else {
-        
+
         prob = enumerate_next(A, i, j, A_running_stat, A_running_prob,
                               A_running_rowsums, A_running_colsums,
                               O_rowsums, O_colsums, O_stat,
                               lb_method, ub_method,
                               & traverse_lt_observed_stat);
     }
-    
+
     return prob;
 }
 
@@ -588,26 +555,26 @@ mydouble exact_func_test_multi_hypergeometric
 {
     mydouble pval=1.0;
     fc = 0;
-    
+
     size_t nrows = O.size();
     size_t ncols = O[0].size();
-    
+
     if (nrows < 2 || ncols < 2) {
         return pval;
     }
-    
+
     int n=0;
-    
+
     vector<vector<int> > A(nrows, vector<int>(ncols, 0));
     mydouble A_running_stat = 0.0;
     mydouble A_running_prob = 1.0;
     vector<vector<int> > A_running_rowsums(A);
     vector<vector<int> > A_running_colsums(A);
-    
+
     // Calculate row and col sums, sample size, and initial probability
     vector<int> O_rowsums(nrows, 0);
     vector<int> O_colsums(ncols, 0);
-    
+
     for (size_t i=0; i<nrows; ++i) {
         for (size_t j=0; j<ncols; ++j) {
             O_rowsums[i] += O[i][j];
@@ -615,7 +582,7 @@ mydouble exact_func_test_multi_hypergeometric
         A_running_prob *= factorial<mydouble>(O_rowsums[i]);
         n += O_rowsums[i];
     }
-    
+
     mydouble ej = n / (mydouble) ncols;
     for (size_t j=0; j<ncols; ++j) {
         for (size_t i=0; i<nrows; ++i) {
@@ -624,11 +591,11 @@ mydouble exact_func_test_multi_hypergeometric
         A_running_prob *= factorial<mydouble>(O_colsums[j]);
         if(ej>0) A_running_stat -= (O_colsums[j]-ej) * (O_colsums[j]-ej) / ej;
     }
-    
+
     A_running_prob /= factorial<mydouble>(n);
-    
+
     fc = funchisq(O, O_rowsums, O_colsums, n);
-    
+
     switch (pval_method) {
         case PVAL:
             pval = traverse_ge_observed_stat
@@ -636,21 +603,21 @@ mydouble exact_func_test_multi_hypergeometric
              A_running_rowsums, A_running_colsums,
              O_rowsums, O_colsums, fc,
              lb_method, ub_method);
-            
+
             break;
-            
+
         case ONE_MINUS:
             pval = 1 - traverse_lt_observed_stat
             (A, 0, 0, A_running_stat, A_running_prob,
              A_running_rowsums, A_running_colsums,
              O_rowsums, O_colsums, fc,
              lb_method, ub_method);
-            
+
             break;
-            
+
         default:
             break;
     }
-    
+
     return pval;
 }
